#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""

"""
from SLiCAP import *

fileName = "feedbackConfigs"
specFileName = "specifications.csv"

# Define an instruction
i1 = instruction()
i1.setCircuit(fileName + '.cir')

# import the specifications
specs = csv2specs(specFileName)

# Define the circuit parameters from the specifications
specs2circuit(specs, i1)

htmlPage("Circuit Data")
head2html("Circuit diagram")
img2html(fileName + ".svg", 1000)
netlist2html(fileName + ".cir")
elementData2html(i1.circuit)
params2html(i1.circuit)
htmlPage('Feedback Configurations')
img2html(fileName + ".svg", 1000)
head2html("Transmit amplifier")
text2html("The transmitter coil is driven from a transadmittance amplifier")
text2html("The selected configuration has a grounded input, and a grounded " +
          "controller output. The transmitter coil is not grounded.")
text2html("This facilitates implementation of the controller with an " +
          "operational amplifier.")
head2html("Receive amplifier")

i1.setSimType("numeric")
f_imax = sp.N(1/i1.getParValue("A_R")/2/sp.pi)
f_max  = i1.getParValue("f_max")
text2html("The receiver is realized with an integrating voltage amplifier " +
          "For frequencies below $f_{imax}=\\frac{1}{2 \\pi A_R}$ this " +
          "amplifier has an integrating character. " +
          "Above this frequency its gain is unity.")

text2html("We have:")
eqn2html("f_imax", f_imax)
text2html("and")
eqn2html("f_max", f_max)

if f_imax >= f_max:
    text2html("Since $f_{imax} \\geq f_{max}$, the zero in the transfer does not " +
              "need to be compensated.")
else:
    text2html("Since $f_{imax} < f_{max}$, the zero in the transfer needs " +
              "to be compensated.")
    
htmlPage("Frequency characteristics")
text2html("The plots below show the frequency characteristics of the voltage " +
          "transfer of the hearing system.")
          
i1.setGainType('gain')
i1.setDataType("laplace")
i1.setSource("V1")
i1.setDetector("V_out")
laplaceResult = i1.execute()
figMagFeedback = plotSweep("figMagFeedback", "Magnitude characteristics", laplaceResult, 10, 1E6, 200, show=True)
figPhaseFeedback = plotSweep("figPhaseFeedback", "Phase characteristics", laplaceResult, 10, 1E6, 200, funcType="phase", show=True)
fig2html(figMagFeedback, 600)
fig2html(figPhaseFeedback, 600)