#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar 14 14:47:05 2023

@author: anton
"""
from SLiCAP import *

# Modeling TLV4111

# DC voltage gain and output resistance (worst case)
A_1 = 10**(90/20) # 90 dB gain at 10 kOhm load
R_1 = 10E4 

A_2 = 10**(88/20)  # 88 dB gain at 100 Ohm load
R_2 = 100

A_v = sp.Symbol('A_v')
R_o = sp.Symbol('R_o')

# Define two equalities (type = sympy.Equality) that write the voltage transfers 
# A_1 and A_2 as a function of the voltage gain and the output resistance of the 
# opamp, and the load resistances R_1 and R_2, respectively.
eq1 = sp.Eq(A_1, A_v*(R_1/(R_1+R_o)))
eq2 = sp.Eq(A_2, A_v*(R_2/(R_2+R_o)))

# Solve these equalities
sol = sp.solve([eq1, eq2])

# Assign the results to variables
A_v = sol[A_v]
R_o = sol[R_o]

# Find the poles from the GB product and the magnitude and phase characteristic
# of the small-signal frequency characteristics
G_B = 2.7E6
f_p2 = 5E6

# Define the SLiCAP opAmp model parameters in text format so they can be stored
# in a library file
Av = str(sp.N(A_v/(1 + ini.Laplace*A_v/2/sp.pi/G_B)/(1+ini.Laplace/2/sp.pi/f_p2),4))
Zo = str(sp.N(R_o, 4))
Cc = "5p"

# Create the library file
lib_text = """TL4111_lib
.model TLV4111 OV av={%s} zo=%s cc=%s\n.end
"""%(Av, Zo, Cc)

# Write lib file
f = open(ini.libraryPath + "TLV4111.lib", "w")
f.write(lib_text)
f.close()

# Create an instruction
i1 = instruction()
# Use the test circuit for the voltage gain
fileName = "opampGainTest"
i1.setCircuit(fileName + ".cir")
htmlPage("Circuit Data")
head2html("Circuit diagram")
img2html(fileName + ".svg", 500, caption="SLiCAP test circuit for measurement " +
         "of the frequency characteristics of the gain of the operational " +
         "amplifier")
netlist2html(fileName + ".cir")
elementData2html(i1.circuit)
params2html(i1.circuit)

# Evaluate the gain for a load resistance of 100 Ohm and 10k Ohm
RL_list = [100, 1E4]

i1.setSimType("numeric")
i1.setGainType("gain")
i1.setDataType("laplace")

i1.setSource("V1")
i1.setDetector("V_out")
i1.setStepList(RL_list)
i1.setStepMethod("list")
i1.setStepVar("R_L")
i1.stepOn()

result = i1.execute()

dBmagFig = plotSweep("dBmagAv", "Magnitude characteristics of the voltage gain", result, 10, 10E6, 200, funcType='dBmag', show=True)
phaseFig = plotSweep("phaseAv", "Phase characteristics of the voltage gain", result, 10, 10E6, 200, funcType='phase', show=True)
head2html("Voltage gain")
fig2html(dBmagFig, 600)
fig2html(phaseFig, 600)

# Use the test circuit for the voltage gain output impedance
fileName = "opampZoutTest"
i1.setCircuit(fileName + ".cir")
htmlPage("Circuit Data")
head2html("Circuit diagram")
img2html(fileName + ".svg", 500, caption="SLiCAP test circuit for measurement " +
         "of the frequency characteristics of the output impedance of the " +
         "operational amplifier")
netlist2html(fileName + ".cir")
elementData2html(i1.circuit)
params2html(i1.circuit)

# Evaluate the output impedance for a voltage gain of 1, 10 and 100
# We will keep the sum of the feedback resistors 10k

i1.setSimType("numeric")
i1.setGainType("gain")
i1.setDataType("laplace")

i1.setSource("I1")
i1.setDetector("V_out")
i1.setStepMethod("array")
i1.setStepVars(["R_1", "R_2"])
feedbackArray = [[0, 9e3, 9.9e3],
                 [1e4, 1e3, 1e2]]
i1.setStepArray(feedbackArray)

result = i1.execute()

magZoFig = plotSweep("magZo", "Magnitude characteristics of the output impedance", result, 10, 10E6, 200, show=True)
head2html("Output impedance")
fig2html(magZoFig, 600)
stepArray2html(i1.stepVars, i1.stepArray)